# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['attr_property']
setup_kwargs = {
    'name': 'attr-property',
    'version': '0.0.2',
    'description': 'Property support for attrs',
    'long_description': '# attr_property\n\n[![Pypi][3]][4] [![Github][5]][6] [![PythonVers][8]][4] [![Travis building][10]][11] [![Codacy][12]][13] [![Codacy coverage][14]][13]\n\nProperty support for attrs\n\n## Installation\n```shell\npip install attr_property\n```\n\n## Usage\n### Definitaion of class with `attr_property`s\n```python\nimport attr\nfrom attr_property import attr_property, attr_property_class\n\n@attr_property_class\n@attr.s\nclass A:\n\n  a = attr_property()\n  b = attr.ib() # compatible with original attr.ib\n```\n\n### Specification of getter, setter, and deleter\n```python\ndef a_getter(this, value):\n    print(\'Property a has been accessed!\')\n    return value + 1\ndef a_setter(this, value):\n    print(f\'Property a has been set with value {value!r}\')\ndef a_deleter(this):\n    print(\'Property has been deleted!\')\n\n@attr_property_class\n@attr.s\nclass A:\n    a = attr_property(getter = a_getter, setter = a_setter, deleter = a_deleter)\n\na = A(a = 1)\n# Property a has been set with value 1\n# Property a has been accessed!\nprint(\'a.a =\', a.a)\n# a.a = 2\ndel a.a\n# Property has been deleted!\na.a\n# Error\n```\n\n### Disabling deleter\n\n```python\n@attr_property_class\n@attr.s\nclass A:\n    a = attr_property(deleter = False)\n\na = A(1)\ndel a.a\n# AttributeError: can\'t delete attribute\n```\n\n### Run `attr.ib`\'s converter and validator in setter\n\n```python\n@attr_property_class\n@attr.s\nclass A:\n    a = attr_property(converter = int, validator_runtime = True, converter_runtime = True)\n\n    @a.validator\n    def lessthan20(self, attribute, value):\n        if value >= 20:\n            raise ValueError("d should be less than 20.")\n\na = A(\'3\')\n# a.a == 3\na.a = \'30\'\n# ValueError\n```\n\nOrder of execution of setter:\n\n- Delete cached value\n- Run converter\n- Run validator\n- Save converted value as raw value\n- Run specified setter\n\n### Caching getter results\n\n```python\n@attr_property_class\n@attr.s\nclass A:\n    a = attr_property(getter = lambda this, value: value + 1, cache = True)\n\na = A(1)\n# a.a == 2\n# will not do value + again\n# validators and converters will be skipped, as well.\n```\n\n### Accessing raw values before getter calculation\n\n```python\n@attr_property_class\n@attr.s\nclass A:\n    a = attr_property(getter = lambda this, value: value + 1, convert  = int, raw = True)\n\na = A(\'1\')\n# a.a == 2\n# a._a == 1 # converted value\na._a = 9\n# AttributeError, it\'s readonly\n```\n\nUsing a different prefix\n```python\n@attr_property_class\n@attr.s\nclass A:\n    a = attr_property(getter = lambda this, value: value + 1, convert  = int, raw = \'raw_\')\n\na = A(\'1\')\n# a.raw_a == 1\n```\n\n## How does it work?\n\n- Hack attrs\' `_attrs_to_init_script` function to insert codes to initiate `self.__attrs_property_raw__` to save raw values and `__attrs_property_cached__` to save cached values.\n- Create `property`s for each attribute in class decorator `attr_property_class`.\n\n[1]: https://github.com/pwwang/attr_property\n[3]: https://img.shields.io/pypi/v/attr_property?style=flat-square\n[4]: https://pypi.org/project/attr_property/\n[5]: https://img.shields.io/github/tag/pwwang/attr_property?style=flat-square\n[6]: https://github.com/pwwang/attr_property\n[8]: https://img.shields.io/pypi/pyversions/attr_property?style=flat-square\n[10]: https://img.shields.io/travis/pwwang/attr_property?style=flat-square\n[11]: https://travis-ci.org/pwwang/attr_property\n[12]: https://img.shields.io/codacy/grade/41140ad263bc435a822777bed8a41b8d?style=flat-square\n[13]: https://app.codacy.com/manual/pwwang/attr_property/dashboard\n[14]: https://img.shields.io/codacy/coverage/41140ad263bc435a822777bed8a41b8d?style=flat-square\n',
    'author': 'pwwang',
    'author_email': 'pwwang@pwwang.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pwwang/attr_property',
    'py_modules': modules,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
